/* jshint latedef:false */
/* jshint forin:false */
/* jshint noempty:false */

// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

'use strict';

var util = require('util');
var azureCommon = require('azure-common');
var Service = azureCommon.Service;
var WebResource = azureCommon.WebResource;

var ApiAppManagementClient = ( /** @lends ApiAppManagementClient */ function() {
  /**
   * @class
   * The Microsoft Azure ApiApp management API provides a RESTful set of web
   * services that interact with Microsoft Azure ApiApp service to manage your
   * ApiApps. The API has entities that capture the relationship between an
   * end user and the Microsoft Azure ApiApp service.  (see
   * http://azure.microsoft.com/en-us/services/app-service/api/ for more
   * information)
   * Initializes a new instance of the ApiAppManagementClient class.
   * @constructor
   * 
   * @param {SubscriptionCloudCredentials} credentials Gets subscription
   * credentials which uniquely identify Microsoft Azure subscription. The
   * subscription ID forms part of the URI for every service call.
   * 
   * @param {string} [credentials.subscriptionId]
   * 
   * @param {string} [baseUri] Gets the URI used as the base for all cloud
   * service requests.
   * 
   * @param {Array} filters
   */
  function ApiAppManagementClient(credentials, baseUri, filters) {
    if (credentials === null || credentials === undefined) {
      throw new Error('credentials cannot be null.');
    }
    
    ApiAppManagementClient['super_'].call(this, credentials, filters);
    
    this.credentials = credentials;
    this.baseUri = baseUri;
    if (this.baseUri === null || this.baseUri === undefined) {
      this.baseUri = 'https://management.azure.com';
    }
    if (this.apiVersion === null || this.apiVersion === undefined) {
      this.apiVersion = '2015-03-01-preview';
    }
    if (this.longRunningOperationInitialTimeout === null || this.longRunningOperationInitialTimeout === undefined) {
      this.longRunningOperationInitialTimeout = -1;
    }
    if (this.longRunningOperationRetryTimeout === null || this.longRunningOperationRetryTimeout === undefined) {
      this.longRunningOperationRetryTimeout = -1;
    }
    /**
     * Provides an instance of the [ApiAppOperations](-ApiAppOperations.html)
     * object.
     * @type {object}
     */
    this.apiApps = new ApiAppOperations(this);
    /**
     * Provides an instance of the
     * [TemplateOperations](-TemplateOperations.html) object.
     * @type {object}
     */
    this.templates = new TemplateOperations(this);
  }
  
  util.inherits(ApiAppManagementClient, Service);
  
  
  return ApiAppManagementClient;
})();
exports.ApiAppManagementClient = ApiAppManagementClient;

var ApiAppOperations = ( /** @lends ApiAppOperations */ function() {
  /**
   * @class
   * ApiApp operations  (see
   * http://azure.microsoft.com/en-us/services/app-service/api/ for more
   * information)
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [ApiAppManagementClient] {@link
   * ApiAppManagementClient~ApiAppManagementClient}.
   * See [apiApps] {@link
   * ApiAppManagementClient~ApiAppManagementClient#apiApps}.
   * Initializes a new instance of the ApiAppOperations class.
   * @constructor
   * 
   * @param {ApiAppManagementClient} client Reference to the service client.
   */
  function ApiAppOperations(client) {
    this.client = client;
  }
  
  /**
   * Get information about a specific ApiApp
   * 
   * @param {string} resourceGroup The resource group containing the api app
   * 
   * @param {string} name The instance name of the apiapp
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ApiAppOperations.prototype.get = function(resourceGroup, name, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroup === null || resourceGroup === undefined) {
      return callback(new Error('resourceGroup cannot be null.'));
    }
    if (name === null || name === undefined) {
      return callback(new Error('name cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourcegroups/';
    url2 = url2 + encodeURIComponent(resourceGroup);
    url2 = url2 + '/providers/Microsoft.AppService/apiapps/';
    url2 = url2 + encodeURIComponent(name);
    var queryParameters = [];
    queryParameters.push('api-version=2015-03-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var apiAppInstance = { dependencies: [], status: [], tags: {} };
          result.apiApp = apiAppInstance;
          
          var typeValue = responseDoc['type'];
          if (typeValue !== null && typeValue !== undefined) {
          }
          
          var idValue = responseDoc['id'];
          if (idValue !== null && idValue !== undefined) {
            var idInstance = idValue;
            apiAppInstance.id = idInstance;
          }
          
          var nameValue = responseDoc['name'];
          if (nameValue !== null && nameValue !== undefined) {
            var nameInstance = nameValue;
            apiAppInstance.name = nameInstance;
          }
          
          var locationValue = responseDoc['location'];
          if (locationValue !== null && locationValue !== undefined) {
            var locationInstance = locationValue;
            apiAppInstance.location = locationInstance;
          }
          
          var tagsSequenceElement = responseDoc['tags'];
          if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
            for (var property in tagsSequenceElement) {
              var tagsKey = property;
              var tagsValue = tagsSequenceElement[property];
              apiAppInstance.tags[tagsKey] = tagsValue;
            }
          }
          
          var propertiesValue = responseDoc['properties'];
          if (propertiesValue !== null && propertiesValue !== undefined) {
            var packageValue = propertiesValue['package'];
            if (packageValue !== null && packageValue !== undefined) {
              var packageInstance = { authentication: [], links: [] };
              apiAppInstance.package = packageInstance;
              
              var idValue2 = packageValue['id'];
              if (idValue2 !== null && idValue2 !== undefined) {
                var idInstance2 = idValue2;
                packageInstance.id = idInstance2;
              }
              
              var versionValue = packageValue['version'];
              if (versionValue !== null && versionValue !== undefined) {
                var versionInstance = versionValue;
                packageInstance.version = versionInstance;
              }
              
              var authenticationArray = packageValue['authentication'];
              if (authenticationArray !== null && authenticationArray !== undefined) {
                for (var loweredIndex1 = 0; loweredIndex1 < authenticationArray.length; loweredIndex1 = loweredIndex1 + 1) {
                  var authenticationValue = authenticationArray[loweredIndex1];
                  var microserviceAuthenticationInstance = { scopes: [] };
                  packageInstance.authentication.push(microserviceAuthenticationInstance);
                  
                  var typeValue2 = authenticationValue['type'];
                  if (typeValue2 !== null && typeValue2 !== undefined) {
                    var typeInstance = typeValue2;
                    microserviceAuthenticationInstance.type = typeInstance;
                  }
                  
                  var scopesArray = authenticationValue['scopes'];
                  if (scopesArray !== null && scopesArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < scopesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var scopesValue = scopesArray[loweredIndex2];
                      microserviceAuthenticationInstance.scopes.push(scopesValue);
                    }
                  }
                }
              }
              
              var linksArray = packageValue['links'];
              if (linksArray !== null && linksArray !== undefined) {
                for (var loweredIndex3 = 0; loweredIndex3 < linksArray.length; loweredIndex3 = loweredIndex3 + 1) {
                  var linksValue = linksArray[loweredIndex3];
                  var microserviceLinkInstance = {};
                  packageInstance.links.push(microserviceLinkInstance);
                  
                  var textValue = linksValue['text'];
                  if (textValue !== null && textValue !== undefined) {
                    var textInstance = textValue;
                    microserviceLinkInstance.text = textInstance;
                  }
                  
                  var urlValue = linksValue['Url'];
                  if (urlValue !== null && urlValue !== undefined) {
                    var urlInstance = urlValue;
                    microserviceLinkInstance.url = urlInstance;
                  }
                }
              }
            }
            
            var updatePolicyValue = propertiesValue['updatePolicy'];
            if (updatePolicyValue !== null && updatePolicyValue !== undefined) {
              var updatePolicyInstance = updatePolicyValue;
              apiAppInstance.updatePolicy = updatePolicyInstance;
            }
            
            var accessLevelValue = propertiesValue['accessLevel'];
            if (accessLevelValue !== null && accessLevelValue !== undefined) {
              var accessLevelInstance = accessLevelValue;
              apiAppInstance.accessLevel = accessLevelInstance;
            }
            
            var apiValue = propertiesValue['api'];
            if (apiValue !== null && apiValue !== undefined) {
              var apiInstance = {};
              apiAppInstance.api = apiInstance;
              
              var endpointValue = apiValue['endpoint'];
              if (endpointValue !== null && endpointValue !== undefined) {
                var endpointInstance = endpointValue;
                apiInstance.endpoint = endpointInstance;
              }
              
              var hasDefinitionValue = apiValue['hasDefinition'];
              if (hasDefinitionValue !== null && hasDefinitionValue !== undefined) {
                var hasDefinitionInstance = hasDefinitionValue;
                apiInstance.hasDefinition = hasDefinitionInstance;
              }
            }
            
            var hostValue = propertiesValue['host'];
            if (hostValue !== null && hostValue !== undefined) {
              var hostInstance = {};
              apiAppInstance.host = hostInstance;
              
              var idValue3 = hostValue['id'];
              if (idValue3 !== null && idValue3 !== undefined) {
                var idInstance3 = idValue3;
                hostInstance.id = idInstance3;
              }
              
              var resourceNameValue = hostValue['resourceName'];
              if (resourceNameValue !== null && resourceNameValue !== undefined) {
                var resourceNameInstance = resourceNameValue;
                hostInstance.resourceName = resourceNameInstance;
              }
              
              var resourceTypeValue = hostValue['resourceType'];
              if (resourceTypeValue !== null && resourceTypeValue !== undefined) {
                var resourceTypeInstance = resourceTypeValue;
                hostInstance.resourceType = resourceTypeInstance;
              }
            }
            
            var gatewayValue = propertiesValue['gateway'];
            if (gatewayValue !== null && gatewayValue !== undefined) {
              var gatewayInstance = {};
              apiAppInstance.gateway = gatewayInstance;
              
              var idValue4 = gatewayValue['id'];
              if (idValue4 !== null && idValue4 !== undefined) {
                var idInstance4 = idValue4;
                gatewayInstance.id = idInstance4;
              }
              
              var resourceNameValue2 = gatewayValue['resourceName'];
              if (resourceNameValue2 !== null && resourceNameValue2 !== undefined) {
                var resourceNameInstance2 = resourceNameValue2;
                gatewayInstance.resourceName = resourceNameInstance2;
              }
              
              var resourceTypeValue2 = gatewayValue['resourceType'];
              if (resourceTypeValue2 !== null && resourceTypeValue2 !== undefined) {
                var resourceTypeInstance2 = resourceTypeValue2;
                gatewayInstance.resourceType = resourceTypeInstance2;
              }
            }
            
            var dependenciesArray = propertiesValue['dependencies'];
            if (dependenciesArray !== null && dependenciesArray !== undefined) {
              for (var loweredIndex4 = 0; loweredIndex4 < dependenciesArray.length; loweredIndex4 = loweredIndex4 + 1) {
                var dependenciesValue = dependenciesArray[loweredIndex4];
                apiAppInstance.dependencies.push(dependenciesValue);
              }
            }
            
            var statusArray = propertiesValue['status'];
            if (statusArray !== null && statusArray !== undefined) {
              for (var loweredIndex5 = 0; loweredIndex5 < statusArray.length; loweredIndex5 = loweredIndex5 + 1) {
                var statusValue = statusArray[loweredIndex5];
                var statusCheckEntryInstance = {};
                apiAppInstance.status.push(statusCheckEntryInstance);
                
                var nameValue2 = statusValue['name'];
                if (nameValue2 !== null && nameValue2 !== undefined) {
                  var nameInstance2 = nameValue2;
                  statusCheckEntryInstance.name = nameInstance2;
                }
                
                var messageValue = statusValue['message'];
                if (messageValue !== null && messageValue !== undefined) {
                  var messageInstance = messageValue;
                  statusCheckEntryInstance.message = messageInstance;
                }
                
                var statusLevelValue = statusValue['statusLevel'];
                if (statusLevelValue !== null && statusLevelValue !== undefined) {
                  var statusLevelInstance = statusLevelValue;
                  statusCheckEntryInstance.statusLevel = statusLevelInstance;
                }
              }
            }
            
            var authenticationValue2 = propertiesValue['authentication'];
            if (authenticationValue2 !== null && authenticationValue2 !== undefined) {
              var authenticationInstance = { scopes: [] };
              apiAppInstance.authentication = authenticationInstance;
              
              var typeValue3 = authenticationValue2['type'];
              if (typeValue3 !== null && typeValue3 !== undefined) {
                var typeInstance2 = typeValue3;
                authenticationInstance.type = typeInstance2;
              }
              
              var scopesArray2 = authenticationValue2['scopes'];
              if (scopesArray2 !== null && scopesArray2 !== undefined) {
                for (var loweredIndex6 = 0; loweredIndex6 < scopesArray2.length; loweredIndex6 = loweredIndex6 + 1) {
                  var scopesValue2 = scopesArray2[loweredIndex6];
                  authenticationInstance.scopes.push(scopesValue2);
                }
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Return list of the deployed ApiApps for the resource group
   * 
   * @param {string} resourceGroup Name of the resource group containing the
   * apiapps to list
   * 
   * @param {string} [expand] Option controlling how much data to return, valid
   * options are "basic" or "detail"
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ApiAppOperations.prototype.list = function(resourceGroup, expand, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (resourceGroup === null || resourceGroup === undefined) {
      return callback(new Error('resourceGroup cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/resourcegroups/';
    url2 = url2 + encodeURIComponent(resourceGroup);
    url2 = url2 + '/providers/Microsoft.AppService/apiapps';
    var queryParameters = [];
    if (expand !== null && expand !== undefined) {
      queryParameters.push('$expand=' + encodeURIComponent(expand));
    }
    queryParameters.push('api-version=2015-03-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { apiApps: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var apiAppJsonInstance = { dependencies: [], status: [], tags: {} };
              result.apiApps.push(apiAppJsonInstance);
              
              var typeValue = valueValue['type'];
              if (typeValue !== null && typeValue !== undefined) {
              }
              
              var idValue = valueValue['id'];
              if (idValue !== null && idValue !== undefined) {
                var idInstance = idValue;
                apiAppJsonInstance.id = idInstance;
              }
              
              var nameValue = valueValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                apiAppJsonInstance.name = nameInstance;
              }
              
              var locationValue = valueValue['location'];
              if (locationValue !== null && locationValue !== undefined) {
                var locationInstance = locationValue;
                apiAppJsonInstance.location = locationInstance;
              }
              
              var tagsSequenceElement = valueValue['tags'];
              if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                for (var property in tagsSequenceElement) {
                  var tagsKey = property;
                  var tagsValue = tagsSequenceElement[property];
                  apiAppJsonInstance.tags[tagsKey] = tagsValue;
                }
              }
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var packageValue = propertiesValue['package'];
                if (packageValue !== null && packageValue !== undefined) {
                  var packageInstance = { authentication: [], links: [] };
                  apiAppJsonInstance.package = packageInstance;
                  
                  var idValue2 = packageValue['id'];
                  if (idValue2 !== null && idValue2 !== undefined) {
                    var idInstance2 = idValue2;
                    packageInstance.id = idInstance2;
                  }
                  
                  var versionValue = packageValue['version'];
                  if (versionValue !== null && versionValue !== undefined) {
                    var versionInstance = versionValue;
                    packageInstance.version = versionInstance;
                  }
                  
                  var authenticationArray = packageValue['authentication'];
                  if (authenticationArray !== null && authenticationArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < authenticationArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var authenticationValue = authenticationArray[loweredIndex2];
                      var microserviceAuthenticationInstance = { scopes: [] };
                      packageInstance.authentication.push(microserviceAuthenticationInstance);
                      
                      var typeValue2 = authenticationValue['type'];
                      if (typeValue2 !== null && typeValue2 !== undefined) {
                        var typeInstance = typeValue2;
                        microserviceAuthenticationInstance.type = typeInstance;
                      }
                      
                      var scopesArray = authenticationValue['scopes'];
                      if (scopesArray !== null && scopesArray !== undefined) {
                        for (var loweredIndex3 = 0; loweredIndex3 < scopesArray.length; loweredIndex3 = loweredIndex3 + 1) {
                          var scopesValue = scopesArray[loweredIndex3];
                          microserviceAuthenticationInstance.scopes.push(scopesValue);
                        }
                      }
                    }
                  }
                  
                  var linksArray = packageValue['links'];
                  if (linksArray !== null && linksArray !== undefined) {
                    for (var loweredIndex4 = 0; loweredIndex4 < linksArray.length; loweredIndex4 = loweredIndex4 + 1) {
                      var linksValue = linksArray[loweredIndex4];
                      var microserviceLinkInstance = {};
                      packageInstance.links.push(microserviceLinkInstance);
                      
                      var textValue = linksValue['text'];
                      if (textValue !== null && textValue !== undefined) {
                        var textInstance = textValue;
                        microserviceLinkInstance.text = textInstance;
                      }
                      
                      var urlValue = linksValue['Url'];
                      if (urlValue !== null && urlValue !== undefined) {
                        var urlInstance = urlValue;
                        microserviceLinkInstance.url = urlInstance;
                      }
                    }
                  }
                }
                
                var updatePolicyValue = propertiesValue['updatePolicy'];
                if (updatePolicyValue !== null && updatePolicyValue !== undefined) {
                  var updatePolicyInstance = updatePolicyValue;
                  apiAppJsonInstance.updatePolicy = updatePolicyInstance;
                }
                
                var accessLevelValue = propertiesValue['accessLevel'];
                if (accessLevelValue !== null && accessLevelValue !== undefined) {
                  var accessLevelInstance = accessLevelValue;
                  apiAppJsonInstance.accessLevel = accessLevelInstance;
                }
                
                var apiValue = propertiesValue['api'];
                if (apiValue !== null && apiValue !== undefined) {
                  var apiInstance = {};
                  apiAppJsonInstance.api = apiInstance;
                  
                  var endpointValue = apiValue['endpoint'];
                  if (endpointValue !== null && endpointValue !== undefined) {
                    var endpointInstance = endpointValue;
                    apiInstance.endpoint = endpointInstance;
                  }
                  
                  var hasDefinitionValue = apiValue['hasDefinition'];
                  if (hasDefinitionValue !== null && hasDefinitionValue !== undefined) {
                    var hasDefinitionInstance = hasDefinitionValue;
                    apiInstance.hasDefinition = hasDefinitionInstance;
                  }
                }
                
                var hostValue = propertiesValue['host'];
                if (hostValue !== null && hostValue !== undefined) {
                  var hostInstance = {};
                  apiAppJsonInstance.host = hostInstance;
                  
                  var idValue3 = hostValue['id'];
                  if (idValue3 !== null && idValue3 !== undefined) {
                    var idInstance3 = idValue3;
                    hostInstance.id = idInstance3;
                  }
                  
                  var resourceNameValue = hostValue['resourceName'];
                  if (resourceNameValue !== null && resourceNameValue !== undefined) {
                    var resourceNameInstance = resourceNameValue;
                    hostInstance.resourceName = resourceNameInstance;
                  }
                  
                  var resourceTypeValue = hostValue['resourceType'];
                  if (resourceTypeValue !== null && resourceTypeValue !== undefined) {
                    var resourceTypeInstance = resourceTypeValue;
                    hostInstance.resourceType = resourceTypeInstance;
                  }
                }
                
                var gatewayValue = propertiesValue['gateway'];
                if (gatewayValue !== null && gatewayValue !== undefined) {
                  var gatewayInstance = {};
                  apiAppJsonInstance.gateway = gatewayInstance;
                  
                  var idValue4 = gatewayValue['id'];
                  if (idValue4 !== null && idValue4 !== undefined) {
                    var idInstance4 = idValue4;
                    gatewayInstance.id = idInstance4;
                  }
                  
                  var resourceNameValue2 = gatewayValue['resourceName'];
                  if (resourceNameValue2 !== null && resourceNameValue2 !== undefined) {
                    var resourceNameInstance2 = resourceNameValue2;
                    gatewayInstance.resourceName = resourceNameInstance2;
                  }
                  
                  var resourceTypeValue2 = gatewayValue['resourceType'];
                  if (resourceTypeValue2 !== null && resourceTypeValue2 !== undefined) {
                    var resourceTypeInstance2 = resourceTypeValue2;
                    gatewayInstance.resourceType = resourceTypeInstance2;
                  }
                }
                
                var dependenciesArray = propertiesValue['dependencies'];
                if (dependenciesArray !== null && dependenciesArray !== undefined) {
                  for (var loweredIndex5 = 0; loweredIndex5 < dependenciesArray.length; loweredIndex5 = loweredIndex5 + 1) {
                    var dependenciesValue = dependenciesArray[loweredIndex5];
                    apiAppJsonInstance.dependencies.push(dependenciesValue);
                  }
                }
                
                var statusArray = propertiesValue['status'];
                if (statusArray !== null && statusArray !== undefined) {
                  for (var loweredIndex6 = 0; loweredIndex6 < statusArray.length; loweredIndex6 = loweredIndex6 + 1) {
                    var statusValue = statusArray[loweredIndex6];
                    var statusCheckEntryInstance = {};
                    apiAppJsonInstance.status.push(statusCheckEntryInstance);
                    
                    var nameValue2 = statusValue['name'];
                    if (nameValue2 !== null && nameValue2 !== undefined) {
                      var nameInstance2 = nameValue2;
                      statusCheckEntryInstance.name = nameInstance2;
                    }
                    
                    var messageValue = statusValue['message'];
                    if (messageValue !== null && messageValue !== undefined) {
                      var messageInstance = messageValue;
                      statusCheckEntryInstance.message = messageInstance;
                    }
                    
                    var statusLevelValue = statusValue['statusLevel'];
                    if (statusLevelValue !== null && statusLevelValue !== undefined) {
                      var statusLevelInstance = statusLevelValue;
                      statusCheckEntryInstance.statusLevel = statusLevelInstance;
                    }
                  }
                }
                
                var authenticationValue2 = propertiesValue['authentication'];
                if (authenticationValue2 !== null && authenticationValue2 !== undefined) {
                  var authenticationInstance = { scopes: [] };
                  apiAppJsonInstance.authentication = authenticationInstance;
                  
                  var typeValue3 = authenticationValue2['type'];
                  if (typeValue3 !== null && typeValue3 !== undefined) {
                    var typeInstance2 = typeValue3;
                    authenticationInstance.type = typeInstance2;
                  }
                  
                  var scopesArray2 = authenticationValue2['scopes'];
                  if (scopesArray2 !== null && scopesArray2 !== undefined) {
                    for (var loweredIndex7 = 0; loweredIndex7 < scopesArray2.length; loweredIndex7 = loweredIndex7 + 1) {
                      var scopesValue2 = scopesArray2[loweredIndex7];
                      authenticationInstance.scopes.push(scopesValue2);
                    }
                  }
                }
              }
            }
          }
          
          var nextLinkValue = responseDoc['nextLink'];
          if (nextLinkValue !== null && nextLinkValue !== undefined) {
            var nextLinkInstance = nextLinkValue;
            result.nextLink = nextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * Return list of the deployed ApiApps for this subscription
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ApiAppOperations.prototype.listAll = function(callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/providers/Microsoft.AppService/apiapps';
    var queryParameters = [];
    queryParameters.push('api-version=2015-03-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = { apiApps: [] };
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var valueArray = responseDoc['value'];
          if (valueArray !== null && valueArray !== undefined) {
            for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
              var valueValue = valueArray[loweredIndex1];
              var apiAppJsonInstance = { dependencies: [], status: [], tags: {} };
              result.apiApps.push(apiAppJsonInstance);
              
              var typeValue = valueValue['type'];
              if (typeValue !== null && typeValue !== undefined) {
              }
              
              var idValue = valueValue['id'];
              if (idValue !== null && idValue !== undefined) {
                var idInstance = idValue;
                apiAppJsonInstance.id = idInstance;
              }
              
              var nameValue = valueValue['name'];
              if (nameValue !== null && nameValue !== undefined) {
                var nameInstance = nameValue;
                apiAppJsonInstance.name = nameInstance;
              }
              
              var locationValue = valueValue['location'];
              if (locationValue !== null && locationValue !== undefined) {
                var locationInstance = locationValue;
                apiAppJsonInstance.location = locationInstance;
              }
              
              var tagsSequenceElement = valueValue['tags'];
              if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
                for (var property in tagsSequenceElement) {
                  var tagsKey = property;
                  var tagsValue = tagsSequenceElement[property];
                  apiAppJsonInstance.tags[tagsKey] = tagsValue;
                }
              }
              
              var propertiesValue = valueValue['properties'];
              if (propertiesValue !== null && propertiesValue !== undefined) {
                var packageValue = propertiesValue['package'];
                if (packageValue !== null && packageValue !== undefined) {
                  var packageInstance = { authentication: [], links: [] };
                  apiAppJsonInstance.package = packageInstance;
                  
                  var idValue2 = packageValue['id'];
                  if (idValue2 !== null && idValue2 !== undefined) {
                    var idInstance2 = idValue2;
                    packageInstance.id = idInstance2;
                  }
                  
                  var versionValue = packageValue['version'];
                  if (versionValue !== null && versionValue !== undefined) {
                    var versionInstance = versionValue;
                    packageInstance.version = versionInstance;
                  }
                  
                  var authenticationArray = packageValue['authentication'];
                  if (authenticationArray !== null && authenticationArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < authenticationArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var authenticationValue = authenticationArray[loweredIndex2];
                      var microserviceAuthenticationInstance = { scopes: [] };
                      packageInstance.authentication.push(microserviceAuthenticationInstance);
                      
                      var typeValue2 = authenticationValue['type'];
                      if (typeValue2 !== null && typeValue2 !== undefined) {
                        var typeInstance = typeValue2;
                        microserviceAuthenticationInstance.type = typeInstance;
                      }
                      
                      var scopesArray = authenticationValue['scopes'];
                      if (scopesArray !== null && scopesArray !== undefined) {
                        for (var loweredIndex3 = 0; loweredIndex3 < scopesArray.length; loweredIndex3 = loweredIndex3 + 1) {
                          var scopesValue = scopesArray[loweredIndex3];
                          microserviceAuthenticationInstance.scopes.push(scopesValue);
                        }
                      }
                    }
                  }
                  
                  var linksArray = packageValue['links'];
                  if (linksArray !== null && linksArray !== undefined) {
                    for (var loweredIndex4 = 0; loweredIndex4 < linksArray.length; loweredIndex4 = loweredIndex4 + 1) {
                      var linksValue = linksArray[loweredIndex4];
                      var microserviceLinkInstance = {};
                      packageInstance.links.push(microserviceLinkInstance);
                      
                      var textValue = linksValue['text'];
                      if (textValue !== null && textValue !== undefined) {
                        var textInstance = textValue;
                        microserviceLinkInstance.text = textInstance;
                      }
                      
                      var urlValue = linksValue['Url'];
                      if (urlValue !== null && urlValue !== undefined) {
                        var urlInstance = urlValue;
                        microserviceLinkInstance.url = urlInstance;
                      }
                    }
                  }
                }
                
                var updatePolicyValue = propertiesValue['updatePolicy'];
                if (updatePolicyValue !== null && updatePolicyValue !== undefined) {
                  var updatePolicyInstance = updatePolicyValue;
                  apiAppJsonInstance.updatePolicy = updatePolicyInstance;
                }
                
                var accessLevelValue = propertiesValue['accessLevel'];
                if (accessLevelValue !== null && accessLevelValue !== undefined) {
                  var accessLevelInstance = accessLevelValue;
                  apiAppJsonInstance.accessLevel = accessLevelInstance;
                }
                
                var apiValue = propertiesValue['api'];
                if (apiValue !== null && apiValue !== undefined) {
                  var apiInstance = {};
                  apiAppJsonInstance.api = apiInstance;
                  
                  var endpointValue = apiValue['endpoint'];
                  if (endpointValue !== null && endpointValue !== undefined) {
                    var endpointInstance = endpointValue;
                    apiInstance.endpoint = endpointInstance;
                  }
                  
                  var hasDefinitionValue = apiValue['hasDefinition'];
                  if (hasDefinitionValue !== null && hasDefinitionValue !== undefined) {
                    var hasDefinitionInstance = hasDefinitionValue;
                    apiInstance.hasDefinition = hasDefinitionInstance;
                  }
                }
                
                var hostValue = propertiesValue['host'];
                if (hostValue !== null && hostValue !== undefined) {
                  var hostInstance = {};
                  apiAppJsonInstance.host = hostInstance;
                  
                  var idValue3 = hostValue['id'];
                  if (idValue3 !== null && idValue3 !== undefined) {
                    var idInstance3 = idValue3;
                    hostInstance.id = idInstance3;
                  }
                  
                  var resourceNameValue = hostValue['resourceName'];
                  if (resourceNameValue !== null && resourceNameValue !== undefined) {
                    var resourceNameInstance = resourceNameValue;
                    hostInstance.resourceName = resourceNameInstance;
                  }
                  
                  var resourceTypeValue = hostValue['resourceType'];
                  if (resourceTypeValue !== null && resourceTypeValue !== undefined) {
                    var resourceTypeInstance = resourceTypeValue;
                    hostInstance.resourceType = resourceTypeInstance;
                  }
                }
                
                var gatewayValue = propertiesValue['gateway'];
                if (gatewayValue !== null && gatewayValue !== undefined) {
                  var gatewayInstance = {};
                  apiAppJsonInstance.gateway = gatewayInstance;
                  
                  var idValue4 = gatewayValue['id'];
                  if (idValue4 !== null && idValue4 !== undefined) {
                    var idInstance4 = idValue4;
                    gatewayInstance.id = idInstance4;
                  }
                  
                  var resourceNameValue2 = gatewayValue['resourceName'];
                  if (resourceNameValue2 !== null && resourceNameValue2 !== undefined) {
                    var resourceNameInstance2 = resourceNameValue2;
                    gatewayInstance.resourceName = resourceNameInstance2;
                  }
                  
                  var resourceTypeValue2 = gatewayValue['resourceType'];
                  if (resourceTypeValue2 !== null && resourceTypeValue2 !== undefined) {
                    var resourceTypeInstance2 = resourceTypeValue2;
                    gatewayInstance.resourceType = resourceTypeInstance2;
                  }
                }
                
                var dependenciesArray = propertiesValue['dependencies'];
                if (dependenciesArray !== null && dependenciesArray !== undefined) {
                  for (var loweredIndex5 = 0; loweredIndex5 < dependenciesArray.length; loweredIndex5 = loweredIndex5 + 1) {
                    var dependenciesValue = dependenciesArray[loweredIndex5];
                    apiAppJsonInstance.dependencies.push(dependenciesValue);
                  }
                }
                
                var statusArray = propertiesValue['status'];
                if (statusArray !== null && statusArray !== undefined) {
                  for (var loweredIndex6 = 0; loweredIndex6 < statusArray.length; loweredIndex6 = loweredIndex6 + 1) {
                    var statusValue = statusArray[loweredIndex6];
                    var statusCheckEntryInstance = {};
                    apiAppJsonInstance.status.push(statusCheckEntryInstance);
                    
                    var nameValue2 = statusValue['name'];
                    if (nameValue2 !== null && nameValue2 !== undefined) {
                      var nameInstance2 = nameValue2;
                      statusCheckEntryInstance.name = nameInstance2;
                    }
                    
                    var messageValue = statusValue['message'];
                    if (messageValue !== null && messageValue !== undefined) {
                      var messageInstance = messageValue;
                      statusCheckEntryInstance.message = messageInstance;
                    }
                    
                    var statusLevelValue = statusValue['statusLevel'];
                    if (statusLevelValue !== null && statusLevelValue !== undefined) {
                      var statusLevelInstance = statusLevelValue;
                      statusCheckEntryInstance.statusLevel = statusLevelInstance;
                    }
                  }
                }
                
                var authenticationValue2 = propertiesValue['authentication'];
                if (authenticationValue2 !== null && authenticationValue2 !== undefined) {
                  var authenticationInstance = { scopes: [] };
                  apiAppJsonInstance.authentication = authenticationInstance;
                  
                  var typeValue3 = authenticationValue2['type'];
                  if (typeValue3 !== null && typeValue3 !== undefined) {
                    var typeInstance2 = typeValue3;
                    authenticationInstance.type = typeInstance2;
                  }
                  
                  var scopesArray2 = authenticationValue2['scopes'];
                  if (scopesArray2 !== null && scopesArray2 !== undefined) {
                    for (var loweredIndex7 = 0; loweredIndex7 < scopesArray2.length; loweredIndex7 = loweredIndex7 + 1) {
                      var scopesValue2 = scopesArray2[loweredIndex7];
                      authenticationInstance.scopes.push(scopesValue2);
                    }
                  }
                }
              }
            }
          }
          
          var nextLinkValue = responseDoc['nextLink'];
          if (nextLinkValue !== null && nextLinkValue !== undefined) {
            var nextLinkInstance = nextLinkValue;
            result.nextLink = nextLinkInstance;
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  /**
   * List the next set of ApiApps from the previous list call.
   * 
   * @param {string} nextLink NextLink from a previous successful call to List
   * operation.
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  ApiAppOperations.prototype.listNext = function(nextLink, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (nextLink === null || nextLink === undefined) {
      return callback(new Error('nextLink cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + nextLink;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'GET';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode >= 400) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      var responseContent = body;
      result = { apiApps: [] };
      var responseDoc = null;
      if (responseContent) {
        responseDoc = JSON.parse(responseContent);
      }
      
      if (responseDoc !== null && responseDoc !== undefined) {
        var valueArray = responseDoc['value'];
        if (valueArray !== null && valueArray !== undefined) {
          for (var loweredIndex1 = 0; loweredIndex1 < valueArray.length; loweredIndex1 = loweredIndex1 + 1) {
            var valueValue = valueArray[loweredIndex1];
            var apiAppJsonInstance = { dependencies: [], status: [], tags: {} };
            result.apiApps.push(apiAppJsonInstance);
            
            var typeValue = valueValue['type'];
            if (typeValue !== null && typeValue !== undefined) {
            }
            
            var idValue = valueValue['id'];
            if (idValue !== null && idValue !== undefined) {
              var idInstance = idValue;
              apiAppJsonInstance.id = idInstance;
            }
            
            var nameValue = valueValue['name'];
            if (nameValue !== null && nameValue !== undefined) {
              var nameInstance = nameValue;
              apiAppJsonInstance.name = nameInstance;
            }
            
            var locationValue = valueValue['location'];
            if (locationValue !== null && locationValue !== undefined) {
              var locationInstance = locationValue;
              apiAppJsonInstance.location = locationInstance;
            }
            
            var tagsSequenceElement = valueValue['tags'];
            if (tagsSequenceElement !== null && tagsSequenceElement !== undefined) {
              for (var property in tagsSequenceElement) {
                var tagsKey = property;
                var tagsValue = tagsSequenceElement[property];
                apiAppJsonInstance.tags[tagsKey] = tagsValue;
              }
            }
            
            var propertiesValue = valueValue['properties'];
            if (propertiesValue !== null && propertiesValue !== undefined) {
              var packageValue = propertiesValue['package'];
              if (packageValue !== null && packageValue !== undefined) {
                var packageInstance = { authentication: [], links: [] };
                apiAppJsonInstance.package = packageInstance;
                
                var idValue2 = packageValue['id'];
                if (idValue2 !== null && idValue2 !== undefined) {
                  var idInstance2 = idValue2;
                  packageInstance.id = idInstance2;
                }
                
                var versionValue = packageValue['version'];
                if (versionValue !== null && versionValue !== undefined) {
                  var versionInstance = versionValue;
                  packageInstance.version = versionInstance;
                }
                
                var authenticationArray = packageValue['authentication'];
                if (authenticationArray !== null && authenticationArray !== undefined) {
                  for (var loweredIndex2 = 0; loweredIndex2 < authenticationArray.length; loweredIndex2 = loweredIndex2 + 1) {
                    var authenticationValue = authenticationArray[loweredIndex2];
                    var microserviceAuthenticationInstance = { scopes: [] };
                    packageInstance.authentication.push(microserviceAuthenticationInstance);
                    
                    var typeValue2 = authenticationValue['type'];
                    if (typeValue2 !== null && typeValue2 !== undefined) {
                      var typeInstance = typeValue2;
                      microserviceAuthenticationInstance.type = typeInstance;
                    }
                    
                    var scopesArray = authenticationValue['scopes'];
                    if (scopesArray !== null && scopesArray !== undefined) {
                      for (var loweredIndex3 = 0; loweredIndex3 < scopesArray.length; loweredIndex3 = loweredIndex3 + 1) {
                        var scopesValue = scopesArray[loweredIndex3];
                        microserviceAuthenticationInstance.scopes.push(scopesValue);
                      }
                    }
                  }
                }
                
                var linksArray = packageValue['links'];
                if (linksArray !== null && linksArray !== undefined) {
                  for (var loweredIndex4 = 0; loweredIndex4 < linksArray.length; loweredIndex4 = loweredIndex4 + 1) {
                    var linksValue = linksArray[loweredIndex4];
                    var microserviceLinkInstance = {};
                    packageInstance.links.push(microserviceLinkInstance);
                    
                    var textValue = linksValue['text'];
                    if (textValue !== null && textValue !== undefined) {
                      var textInstance = textValue;
                      microserviceLinkInstance.text = textInstance;
                    }
                    
                    var urlValue = linksValue['Url'];
                    if (urlValue !== null && urlValue !== undefined) {
                      var urlInstance = urlValue;
                      microserviceLinkInstance.url = urlInstance;
                    }
                  }
                }
              }
              
              var updatePolicyValue = propertiesValue['updatePolicy'];
              if (updatePolicyValue !== null && updatePolicyValue !== undefined) {
                var updatePolicyInstance = updatePolicyValue;
                apiAppJsonInstance.updatePolicy = updatePolicyInstance;
              }
              
              var accessLevelValue = propertiesValue['accessLevel'];
              if (accessLevelValue !== null && accessLevelValue !== undefined) {
                var accessLevelInstance = accessLevelValue;
                apiAppJsonInstance.accessLevel = accessLevelInstance;
              }
              
              var apiValue = propertiesValue['api'];
              if (apiValue !== null && apiValue !== undefined) {
                var apiInstance = {};
                apiAppJsonInstance.api = apiInstance;
                
                var endpointValue = apiValue['endpoint'];
                if (endpointValue !== null && endpointValue !== undefined) {
                  var endpointInstance = endpointValue;
                  apiInstance.endpoint = endpointInstance;
                }
                
                var hasDefinitionValue = apiValue['hasDefinition'];
                if (hasDefinitionValue !== null && hasDefinitionValue !== undefined) {
                  var hasDefinitionInstance = hasDefinitionValue;
                  apiInstance.hasDefinition = hasDefinitionInstance;
                }
              }
              
              var hostValue = propertiesValue['host'];
              if (hostValue !== null && hostValue !== undefined) {
                var hostInstance = {};
                apiAppJsonInstance.host = hostInstance;
                
                var idValue3 = hostValue['id'];
                if (idValue3 !== null && idValue3 !== undefined) {
                  var idInstance3 = idValue3;
                  hostInstance.id = idInstance3;
                }
                
                var resourceNameValue = hostValue['resourceName'];
                if (resourceNameValue !== null && resourceNameValue !== undefined) {
                  var resourceNameInstance = resourceNameValue;
                  hostInstance.resourceName = resourceNameInstance;
                }
                
                var resourceTypeValue = hostValue['resourceType'];
                if (resourceTypeValue !== null && resourceTypeValue !== undefined) {
                  var resourceTypeInstance = resourceTypeValue;
                  hostInstance.resourceType = resourceTypeInstance;
                }
              }
              
              var gatewayValue = propertiesValue['gateway'];
              if (gatewayValue !== null && gatewayValue !== undefined) {
                var gatewayInstance = {};
                apiAppJsonInstance.gateway = gatewayInstance;
                
                var idValue4 = gatewayValue['id'];
                if (idValue4 !== null && idValue4 !== undefined) {
                  var idInstance4 = idValue4;
                  gatewayInstance.id = idInstance4;
                }
                
                var resourceNameValue2 = gatewayValue['resourceName'];
                if (resourceNameValue2 !== null && resourceNameValue2 !== undefined) {
                  var resourceNameInstance2 = resourceNameValue2;
                  gatewayInstance.resourceName = resourceNameInstance2;
                }
                
                var resourceTypeValue2 = gatewayValue['resourceType'];
                if (resourceTypeValue2 !== null && resourceTypeValue2 !== undefined) {
                  var resourceTypeInstance2 = resourceTypeValue2;
                  gatewayInstance.resourceType = resourceTypeInstance2;
                }
              }
              
              var dependenciesArray = propertiesValue['dependencies'];
              if (dependenciesArray !== null && dependenciesArray !== undefined) {
                for (var loweredIndex5 = 0; loweredIndex5 < dependenciesArray.length; loweredIndex5 = loweredIndex5 + 1) {
                  var dependenciesValue = dependenciesArray[loweredIndex5];
                  apiAppJsonInstance.dependencies.push(dependenciesValue);
                }
              }
              
              var statusArray = propertiesValue['status'];
              if (statusArray !== null && statusArray !== undefined) {
                for (var loweredIndex6 = 0; loweredIndex6 < statusArray.length; loweredIndex6 = loweredIndex6 + 1) {
                  var statusValue = statusArray[loweredIndex6];
                  var statusCheckEntryInstance = {};
                  apiAppJsonInstance.status.push(statusCheckEntryInstance);
                  
                  var nameValue2 = statusValue['name'];
                  if (nameValue2 !== null && nameValue2 !== undefined) {
                    var nameInstance2 = nameValue2;
                    statusCheckEntryInstance.name = nameInstance2;
                  }
                  
                  var messageValue = statusValue['message'];
                  if (messageValue !== null && messageValue !== undefined) {
                    var messageInstance = messageValue;
                    statusCheckEntryInstance.message = messageInstance;
                  }
                  
                  var statusLevelValue = statusValue['statusLevel'];
                  if (statusLevelValue !== null && statusLevelValue !== undefined) {
                    var statusLevelInstance = statusLevelValue;
                    statusCheckEntryInstance.statusLevel = statusLevelInstance;
                  }
                }
              }
              
              var authenticationValue2 = propertiesValue['authentication'];
              if (authenticationValue2 !== null && authenticationValue2 !== undefined) {
                var authenticationInstance = { scopes: [] };
                apiAppJsonInstance.authentication = authenticationInstance;
                
                var typeValue3 = authenticationValue2['type'];
                if (typeValue3 !== null && typeValue3 !== undefined) {
                  var typeInstance2 = typeValue3;
                  authenticationInstance.type = typeInstance2;
                }
                
                var scopesArray2 = authenticationValue2['scopes'];
                if (scopesArray2 !== null && scopesArray2 !== undefined) {
                  for (var loweredIndex7 = 0; loweredIndex7 < scopesArray2.length; loweredIndex7 = loweredIndex7 + 1) {
                    var scopesValue2 = scopesArray2[loweredIndex7];
                    authenticationInstance.scopes.push(scopesValue2);
                  }
                }
              }
            }
          }
        }
        
        var nextLinkValue = responseDoc['nextLink'];
        if (nextLinkValue !== null && nextLinkValue !== undefined) {
          var nextLinkInstance = nextLinkValue;
          result.nextLink = nextLinkInstance;
        }
      }
      
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return ApiAppOperations;
})();

var TemplateOperations = ( /** @lends TemplateOperations */ function() {
  /**
   * @class
   * Operations for generating deployment templates
   * __NOTE__: An instance of this class is automatically created for an
   * instance of the [ApiAppManagementClient] {@link
   * ApiAppManagementClient~ApiAppManagementClient}.
   * See [templates] {@link
   * ApiAppManagementClient~ApiAppManagementClient#templates}.
   * Initializes a new instance of the TemplateOperations class.
   * @constructor
   * 
   * @param {ApiAppManagementClient} client Reference to the service client.
   */
  function TemplateOperations(client) {
    this.client = client;
  }
  
  /**
   * Get metadata for deploying a package to the given resource group
   * 
   * @param {GetDeploymentMetadataRequest} parameters Parameters for operation
   * 
   * @param {string} [parameters.microserviceId] Gets or sets service id
   * 
   * @param {string} [parameters.resourceGroupName] Gets or sets resource group
   * 
   * @param {function} callback
   * 
   * @returns {Stream} The response stream.
   */
  TemplateOperations.prototype.getMetadata = function(parameters, callback) {
    if (callback === null || callback === undefined) {
      throw new Error('callback cannot be null.');
    }
    // Validate
    if (parameters === null || parameters === undefined) {
      return callback(new Error('parameters cannot be null.'));
    }
    
    // Tracing
    
    // Construct URL
    var url2 = '';
    url2 = url2 + '/subscriptions/';
    if (this.client.credentials.subscriptionId !== null && this.client.credentials.subscriptionId !== undefined) {
      url2 = url2 + encodeURIComponent(this.client.credentials.subscriptionId);
    }
    url2 = url2 + '/providers/Microsoft.AppService/deploymenttemplates/';
    if (parameters.microserviceId !== null && parameters.microserviceId !== undefined) {
      url2 = url2 + encodeURIComponent(parameters.microserviceId);
    }
    url2 = url2 + '/listmetadata';
    var queryParameters = [];
    queryParameters.push('api-version=2015-03-01-preview');
    if (queryParameters.length > 0) {
      url2 = url2 + '?' + queryParameters.join('&');
    }
    var baseUrl = this.client.baseUri;
    // Trim '/' character from the end of baseUrl and beginning of url.
    if (baseUrl[baseUrl.length - 1] === '/') {
      baseUrl = baseUrl.substring(0, (baseUrl.length - 1) + 0);
    }
    if (url2[0] === '/') {
      url2 = url2.substring(1);
    }
    url2 = baseUrl + '/' + url2;
    url2 = url2.replace(' ', '%20');
    
    // Create HTTP transport objects
    var httpRequest = new WebResource();
    httpRequest.method = 'POST';
    httpRequest.headers = {};
    httpRequest.url = url2;
    
    // Set Headers
    httpRequest.headers['Content-Type'] = 'application/json';
    
    // Send Request
    return this.client.pipeline(httpRequest, function (err, response, body) {
      if (err !== null && err !== undefined) {
        return callback(err);
      }
      var statusCode = response.statusCode;
      if (statusCode !== 200) {
        var error = new Error(body);
        error.statusCode = response.statusCode;
        return callback(error);
      }
      
      // Create Result
      var result = null;
      // Deserialize Response
      if (statusCode === 200) {
        var responseContent = body;
        result = {};
        var responseDoc = null;
        if (responseContent) {
          responseDoc = JSON.parse(responseContent);
        }
        
        if (responseDoc !== null && responseDoc !== undefined) {
          var metadataInstance = { dependsOn: [], parameters: [] };
          result.metadata = metadataInstance;
          
          var valueValue = responseDoc['value'];
          if (valueValue !== null && valueValue !== undefined) {
            var microserviceIdValue = valueValue['microserviceId'];
            if (microserviceIdValue !== null && microserviceIdValue !== undefined) {
              var microserviceIdInstance = microserviceIdValue;
              metadataInstance.microserviceId = microserviceIdInstance;
            }
            
            var displayNameValue = valueValue['displayName'];
            if (displayNameValue !== null && displayNameValue !== undefined) {
              var displayNameInstance = displayNameValue;
              metadataInstance.displayName = displayNameInstance;
            }
            
            var appSettingsArray = valueValue['appSettings'];
            if (appSettingsArray !== null && appSettingsArray !== undefined) {
              for (var loweredIndex1 = 0; loweredIndex1 < appSettingsArray.length; loweredIndex1 = loweredIndex1 + 1) {
                var appSettingsValue = appSettingsArray[loweredIndex1];
                var parameterMetadataInstance = {};
                metadataInstance.parameters.push(parameterMetadataInstance);
                
                var nameValue = appSettingsValue['name'];
                if (nameValue !== null && nameValue !== undefined) {
                  var nameInstance = nameValue;
                  parameterMetadataInstance.name = nameInstance;
                }
                
                var typeValue = appSettingsValue['type'];
                if (typeValue !== null && typeValue !== undefined) {
                  var typeInstance = typeValue;
                  parameterMetadataInstance.type = typeInstance;
                }
                
                var displayNameValue2 = appSettingsValue['displayName'];
                if (displayNameValue2 !== null && displayNameValue2 !== undefined) {
                  var displayNameInstance2 = displayNameValue2;
                  parameterMetadataInstance.displayName = displayNameInstance2;
                }
                
                var descriptionValue = appSettingsValue['description'];
                if (descriptionValue !== null && descriptionValue !== undefined) {
                  var descriptionInstance = descriptionValue;
                  parameterMetadataInstance.description = descriptionInstance;
                }
                
                var tooltipValue = appSettingsValue['tooltip'];
                if (tooltipValue !== null && tooltipValue !== undefined) {
                  var tooltipInstance = tooltipValue;
                  parameterMetadataInstance.tooltip = tooltipInstance;
                }
                
                var uiHintValue = appSettingsValue['uiHint'];
                if (uiHintValue !== null && uiHintValue !== undefined) {
                  var uiHintInstance = uiHintValue;
                  parameterMetadataInstance.uIHint = uiHintInstance;
                }
                
                var defaultValueValue = appSettingsValue['defaultValue'];
                if (defaultValueValue !== null && defaultValueValue !== undefined) {
                  var defaultValueInstance = defaultValueValue;
                  parameterMetadataInstance.defaultValue = defaultValueInstance;
                }
                
                var constraintsValue = appSettingsValue['constraints'];
                if (constraintsValue !== null && constraintsValue !== undefined) {
                  var constraintsInstance = { allowedValues: [] };
                  parameterMetadataInstance.constraints = constraintsInstance;
                  
                  var requiredValue = constraintsValue['required'];
                  if (requiredValue !== null && requiredValue !== undefined) {
                    var requiredInstance = requiredValue;
                    constraintsInstance.required = requiredInstance;
                  }
                  
                  var hiddenValue = constraintsValue['hidden'];
                  if (hiddenValue !== null && hiddenValue !== undefined) {
                    var hiddenInstance = hiddenValue;
                    constraintsInstance.hidden = hiddenInstance;
                  }
                  
                  var allowedValuesArray = constraintsValue['allowedValues'];
                  if (allowedValuesArray !== null && allowedValuesArray !== undefined) {
                    for (var loweredIndex2 = 0; loweredIndex2 < allowedValuesArray.length; loweredIndex2 = loweredIndex2 + 1) {
                      var allowedValuesValue = allowedValuesArray[loweredIndex2];
                      constraintsInstance.allowedValues.push(allowedValuesValue);
                    }
                  }
                  
                  var rangeValue = constraintsValue['range'];
                  if (rangeValue !== null && rangeValue !== undefined) {
                    var rangeInstance = {};
                    constraintsInstance.range = rangeInstance;
                    
                    var lowerBoundValue = rangeValue['lowerBound'];
                    if (lowerBoundValue !== null && lowerBoundValue !== undefined) {
                      var lowerBoundInstance = lowerBoundValue;
                      rangeInstance.lowerBound = lowerBoundInstance;
                    }
                    
                    var upperBoundValue = rangeValue['upperBound'];
                    if (upperBoundValue !== null && upperBoundValue !== undefined) {
                      var upperBoundInstance = upperBoundValue;
                      rangeInstance.upperBound = upperBoundInstance;
                    }
                  }
                  
                  var lengthValue = constraintsValue['length'];
                  if (lengthValue !== null && lengthValue !== undefined) {
                    var lengthInstance = {};
                    constraintsInstance.length = lengthInstance;
                    
                    var minValue = lengthValue['min'];
                    if (minValue !== null && minValue !== undefined) {
                      var minInstance = minValue;
                      lengthInstance.min = minInstance;
                    }
                    
                    var maxValue = lengthValue['max'];
                    if (maxValue !== null && maxValue !== undefined) {
                      var maxInstance = maxValue;
                      lengthInstance.max = maxInstance;
                    }
                  }
                  
                  var containsCharactersValue = constraintsValue['containsCharacters'];
                  if (containsCharactersValue !== null && containsCharactersValue !== undefined) {
                    var containsCharactersInstance = containsCharactersValue;
                    constraintsInstance.containsCharacters = containsCharactersInstance;
                  }
                  
                  var notContainsCharactersValue = constraintsValue['notContainsCharacters'];
                  if (notContainsCharactersValue !== null && notContainsCharactersValue !== undefined) {
                    var notContainsCharactersInstance = notContainsCharactersValue;
                    constraintsInstance.notContainsCharacters = notContainsCharactersInstance;
                  }
                  
                  var hasDigitValue = constraintsValue['hasDigit'];
                  if (hasDigitValue !== null && hasDigitValue !== undefined) {
                    var hasDigitInstance = hasDigitValue;
                    constraintsInstance.hasDigit = hasDigitInstance;
                  }
                  
                  var hasLetterValue = constraintsValue['hasLetter'];
                  if (hasLetterValue !== null && hasLetterValue !== undefined) {
                    var hasLetterInstance = hasLetterValue;
                    constraintsInstance.hasLetter = hasLetterInstance;
                  }
                  
                  var hasPunctuationValue = constraintsValue['hasPunctuation'];
                  if (hasPunctuationValue !== null && hasPunctuationValue !== undefined) {
                    var hasPunctuationInstance = hasPunctuationValue;
                    constraintsInstance.hasPunctuation = hasPunctuationInstance;
                  }
                  
                  var numericValue = constraintsValue['numeric'];
                  if (numericValue !== null && numericValue !== undefined) {
                    var numericInstance = numericValue;
                    constraintsInstance.numeric = numericInstance;
                  }
                }
              }
            }
            
            var dependsOnArray = valueValue['dependsOn'];
            if (dependsOnArray !== null && dependsOnArray !== undefined) {
              for (var loweredIndex3 = 0; loweredIndex3 < dependsOnArray.length; loweredIndex3 = loweredIndex3 + 1) {
                var dependsOnValue = dependsOnArray[loweredIndex3];
                var microserviceMetadataInstance = { parameters: [] };
                metadataInstance.dependsOn.push(microserviceMetadataInstance);
                
                var microserviceIdValue2 = dependsOnValue['microserviceId'];
                if (microserviceIdValue2 !== null && microserviceIdValue2 !== undefined) {
                  var microserviceIdInstance2 = microserviceIdValue2;
                  microserviceMetadataInstance.microserviceId = microserviceIdInstance2;
                }
                
                var displayNameValue3 = dependsOnValue['displayName'];
                if (displayNameValue3 !== null && displayNameValue3 !== undefined) {
                  var displayNameInstance3 = displayNameValue3;
                  microserviceMetadataInstance.displayName = displayNameInstance3;
                }
                
                var appSettingsArray2 = dependsOnValue['appSettings'];
                if (appSettingsArray2 !== null && appSettingsArray2 !== undefined) {
                  for (var loweredIndex4 = 0; loweredIndex4 < appSettingsArray2.length; loweredIndex4 = loweredIndex4 + 1) {
                    var appSettingsValue2 = appSettingsArray2[loweredIndex4];
                    var parameterMetadataInstance2 = {};
                    microserviceMetadataInstance.parameters.push(parameterMetadataInstance2);
                    
                    var nameValue2 = appSettingsValue2['name'];
                    if (nameValue2 !== null && nameValue2 !== undefined) {
                      var nameInstance2 = nameValue2;
                      parameterMetadataInstance2.name = nameInstance2;
                    }
                    
                    var typeValue2 = appSettingsValue2['type'];
                    if (typeValue2 !== null && typeValue2 !== undefined) {
                      var typeInstance2 = typeValue2;
                      parameterMetadataInstance2.type = typeInstance2;
                    }
                    
                    var displayNameValue4 = appSettingsValue2['displayName'];
                    if (displayNameValue4 !== null && displayNameValue4 !== undefined) {
                      var displayNameInstance4 = displayNameValue4;
                      parameterMetadataInstance2.displayName = displayNameInstance4;
                    }
                    
                    var descriptionValue2 = appSettingsValue2['description'];
                    if (descriptionValue2 !== null && descriptionValue2 !== undefined) {
                      var descriptionInstance2 = descriptionValue2;
                      parameterMetadataInstance2.description = descriptionInstance2;
                    }
                    
                    var tooltipValue2 = appSettingsValue2['tooltip'];
                    if (tooltipValue2 !== null && tooltipValue2 !== undefined) {
                      var tooltipInstance2 = tooltipValue2;
                      parameterMetadataInstance2.tooltip = tooltipInstance2;
                    }
                    
                    var uiHintValue2 = appSettingsValue2['uiHint'];
                    if (uiHintValue2 !== null && uiHintValue2 !== undefined) {
                      var uiHintInstance2 = uiHintValue2;
                      parameterMetadataInstance2.uIHint = uiHintInstance2;
                    }
                    
                    var defaultValueValue2 = appSettingsValue2['defaultValue'];
                    if (defaultValueValue2 !== null && defaultValueValue2 !== undefined) {
                      var defaultValueInstance2 = defaultValueValue2;
                      parameterMetadataInstance2.defaultValue = defaultValueInstance2;
                    }
                    
                    var constraintsValue2 = appSettingsValue2['constraints'];
                    if (constraintsValue2 !== null && constraintsValue2 !== undefined) {
                      var constraintsInstance2 = { allowedValues: [] };
                      parameterMetadataInstance2.constraints = constraintsInstance2;
                      
                      var requiredValue2 = constraintsValue2['required'];
                      if (requiredValue2 !== null && requiredValue2 !== undefined) {
                        var requiredInstance2 = requiredValue2;
                        constraintsInstance2.required = requiredInstance2;
                      }
                      
                      var hiddenValue2 = constraintsValue2['hidden'];
                      if (hiddenValue2 !== null && hiddenValue2 !== undefined) {
                        var hiddenInstance2 = hiddenValue2;
                        constraintsInstance2.hidden = hiddenInstance2;
                      }
                      
                      var allowedValuesArray2 = constraintsValue2['allowedValues'];
                      if (allowedValuesArray2 !== null && allowedValuesArray2 !== undefined) {
                        for (var loweredIndex5 = 0; loweredIndex5 < allowedValuesArray2.length; loweredIndex5 = loweredIndex5 + 1) {
                          var allowedValuesValue2 = allowedValuesArray2[loweredIndex5];
                          constraintsInstance2.allowedValues.push(allowedValuesValue2);
                        }
                      }
                      
                      var rangeValue2 = constraintsValue2['range'];
                      if (rangeValue2 !== null && rangeValue2 !== undefined) {
                        var rangeInstance2 = {};
                        constraintsInstance2.range = rangeInstance2;
                        
                        var lowerBoundValue2 = rangeValue2['lowerBound'];
                        if (lowerBoundValue2 !== null && lowerBoundValue2 !== undefined) {
                          var lowerBoundInstance2 = lowerBoundValue2;
                          rangeInstance2.lowerBound = lowerBoundInstance2;
                        }
                        
                        var upperBoundValue2 = rangeValue2['upperBound'];
                        if (upperBoundValue2 !== null && upperBoundValue2 !== undefined) {
                          var upperBoundInstance2 = upperBoundValue2;
                          rangeInstance2.upperBound = upperBoundInstance2;
                        }
                      }
                      
                      var lengthValue2 = constraintsValue2['length'];
                      if (lengthValue2 !== null && lengthValue2 !== undefined) {
                        var lengthInstance2 = {};
                        constraintsInstance2.length = lengthInstance2;
                        
                        var minValue2 = lengthValue2['min'];
                        if (minValue2 !== null && minValue2 !== undefined) {
                          var minInstance2 = minValue2;
                          lengthInstance2.min = minInstance2;
                        }
                        
                        var maxValue2 = lengthValue2['max'];
                        if (maxValue2 !== null && maxValue2 !== undefined) {
                          var maxInstance2 = maxValue2;
                          lengthInstance2.max = maxInstance2;
                        }
                      }
                      
                      var containsCharactersValue2 = constraintsValue2['containsCharacters'];
                      if (containsCharactersValue2 !== null && containsCharactersValue2 !== undefined) {
                        var containsCharactersInstance2 = containsCharactersValue2;
                        constraintsInstance2.containsCharacters = containsCharactersInstance2;
                      }
                      
                      var notContainsCharactersValue2 = constraintsValue2['notContainsCharacters'];
                      if (notContainsCharactersValue2 !== null && notContainsCharactersValue2 !== undefined) {
                        var notContainsCharactersInstance2 = notContainsCharactersValue2;
                        constraintsInstance2.notContainsCharacters = notContainsCharactersInstance2;
                      }
                      
                      var hasDigitValue2 = constraintsValue2['hasDigit'];
                      if (hasDigitValue2 !== null && hasDigitValue2 !== undefined) {
                        var hasDigitInstance2 = hasDigitValue2;
                        constraintsInstance2.hasDigit = hasDigitInstance2;
                      }
                      
                      var hasLetterValue2 = constraintsValue2['hasLetter'];
                      if (hasLetterValue2 !== null && hasLetterValue2 !== undefined) {
                        var hasLetterInstance2 = hasLetterValue2;
                        constraintsInstance2.hasLetter = hasLetterInstance2;
                      }
                      
                      var hasPunctuationValue2 = constraintsValue2['hasPunctuation'];
                      if (hasPunctuationValue2 !== null && hasPunctuationValue2 !== undefined) {
                        var hasPunctuationInstance2 = hasPunctuationValue2;
                        constraintsInstance2.hasPunctuation = hasPunctuationInstance2;
                      }
                      
                      var numericValue2 = constraintsValue2['numeric'];
                      if (numericValue2 !== null && numericValue2 !== undefined) {
                        var numericInstance2 = numericValue2;
                        constraintsInstance2.numeric = numericInstance2;
                      }
                    }
                  }
                }
              }
            }
          }
        }
        
      }
      result.statusCode = statusCode;
      result.requestId = response.headers['x-ms-request-id'];
      
      return callback(null, result);
    });
  };
  
  return TemplateOperations;
})();
